#include "m50530.h"
#include <util/delay.h>
#include <avr/pgmspace.h>

/*
 * LCD signalisieren, dass Daten da sind und übertragen.
 */
#define LCDToggleEX					\
	asm("nop");						\
									\
	LCDCTRLPORT |= _BV(LCDPIN_EX);	\
									\
	asm("nop");						\
	asm("nop");						\
	asm("nop");						\
									\
	LCDCTRLPORT &= ~_BV(LCDPIN_EX);	\
									\
	asm("nop");						\
	asm("nop");						\
	asm("nop");


void LCDSetDataPort(unsigned char data) {
	LCDDATAPORT0 &= ~(_BV(LCDDATABIT0));
	LCDDATAPORT1 &= ~(_BV(LCDDATABIT1));
	LCDDATAPORT2 &= ~(_BV(LCDDATABIT2));
	LCDDATAPORT3 &= ~(_BV(LCDDATABIT3));
	LCDDATAPORT0 |= (data & 0x1) << LCDDATABIT0;
	LCDDATAPORT1 |= ((data >> 1) & 0x1) << LCDDATABIT1;
	LCDDATAPORT2 |= ((data >> 2) & 0x1) << LCDDATABIT2;
	LCDDATAPORT3 |= ((data >> 3) & 0x1) << LCDDATABIT3;
}

void LCDSetDataDir(unsigned char data) {
	LCDDATADIR0 &= ~(_BV(LCDDATABIT0));
	LCDDATADIR1 &= ~(_BV(LCDDATABIT1));
	LCDDATADIR2 &= ~(_BV(LCDDATABIT2));
	LCDDATADIR3 &= ~(_BV(LCDDATABIT3));
	LCDDATADIR0 |= (data & 0x1) << LCDDATABIT0;
	LCDDATADIR1 |= ((data >> 1) & 0x1) << LCDDATABIT1;
	LCDDATADIR2 |= ((data >> 2) & 0x1) << LCDDATABIT2;
	LCDDATADIR3 |= ((data >> 3) & 0x1) << LCDDATABIT3;
}

unsigned char LCDGetDataIn() {
	unsigned char ret = 0x00;

	ret |= (LCDDATAPIN0 & _BV(LCDDATABIT0)) >> LCDDATABIT0;
	ret |= ((LCDDATAPIN1 & _BV(LCDDATABIT1)) >> LCDDATABIT1) << 1;
	ret |= ((LCDDATAPIN2 & _BV(LCDDATABIT2)) >> LCDDATABIT2) << 2;
	ret |= ((LCDDATAPIN3 & _BV(LCDDATABIT3)) >> LCDDATABIT3) << 3;

	return ret;
}

/*
 * AControl
 * 		Daten für das Control-Register
 * AData
 * 		8 Bit Daten, die gesendet werden sollen.
 */
void LCDSend(unsigned char AControl, unsigned char AData) {
	/* Wir müssen AData in die oberen und unteren 4 Bit splitten
	 * um diese nacheinander übertragen zu können.
	 */
	unsigned char lHN = (AData & 0xf0) >> 4;
	unsigned char lLN = (AData & 0x0f);

	// Warten, bis das LCD bereit ist Daten zu empfangen.
	unsigned char i;
	for (i = 0; i < 40; i++) 	// eventuell hier den Wert anpassen
		asm("nop");

	// Alle Bits vom Control-Port schließen.
	LCDCTRLPORT &= ~(_BV(LCDPIN_RW) | _BV(LCDPIN_IOC1) | _BV(LCDPIN_IOC2)
			| _BV(LCDPIN_EX));

	// Control-Bits setzen.
	LCDCTRLPORT |= AControl;

	// Obere 4 Bit an den Data-Port legen.
	LCDSetDataPort(lHN);

	// 4 Bit reinschieben.
	LCDToggleEX;

	// Untere 4 Bit an den Data-Port legen.
	LCDSetDataPort(lLN);

	// 4 Bit reinschieben.
	LCDToggleEX;

	// Daten-Port nullen.
	LCDSetDataPort(0x0);

	// Control-Bits zurücksetzen.
	LCDCTRLPORT &= ~AControl;
}

void LCDInit(void) {
	// port setup
	LCDSetDataDir(0x0f);
	LCDCTRLPORT = 0x00;
	LCDCTRLDIR |= _BV(LCDPIN_RW) | _BV(LCDPIN_IOC1) | _BV(LCDPIN_IOC2)
			| _BV(LCDPIN_EX);

	// SF - Set Function Mode
	//  >> 4 Bit I/O, 5x8 Font, 1/32 duty (corresponding for 4 lines)
	//	>> 4 lines x 40 words, 96 words CG RAM
	LCDSend(0, LCD_CMD_SETFUNCTIONMODE | sfIO4BIT | sfFont5x8 | 0x0B);

	// SE - Set Entry Mode
	LCDSend(0, LCD_CMD_SETENTRYMODE | emCursorIncWrite );

	// SU - Set Underline Mode
	LCDSend(0, LCD_CMD_SETUNDERLINEMODE | suNoUnderline );

	// SB - Set Blinking Frequency
	LCDSend(0, LCD_CMD_SETBLINKINGFREQUENCY | sbFastest );

	// SD - Set Display Mode
	LCDSend(0, LCD_CMD_SETDISPLAYMODE | dmDisplayOn
			/*| dmCursorOn */ | dmCursorBlink);

	// CH - Clear Cursor of DD RAM display data home & display start address home
	LCDSend(0, LCD_CMD_CLEARCURSORDATAADDRHOME );
}

// setzt den Cursor an die angegebene Position (y = Zeile, x = Spalte)
void LCDSetCursorPos(const unsigned char y, const unsigned char x) {
	// Adresse aus x und y berechnen
	unsigned char address = x + (y * 40);

	LCDSend(_BV(LCDPIN_IOC1) | _BV(LCDPIN_IOC2), address);
}

// schreibt die übergebene Zeichenkette an die aktuelle Cursor-Position
void LCDWrite(const char * c) {
	while (*c)
		LCDWriteChar(*c++);
}

// schreibt die übergebene Zeichenkette aus dem Programmspeicher
// an die aktuelle Cursor-Position
void LCDWritePGM(const char * c) {
	uint8_t b;

	while ((b = pgm_read_byte(c++)))
		LCDWriteChar(b);
}
